@@ -1,12 +1,10 @@
 #!/usr/bin/env python
-# coding: utf-8
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 
 # Standard library imports
 import os
-import subprocess
 
 # Constants
 HERE = os.path.abspath(os.path.dirname(__file__))
