@@ -6,13 +6,13 @@
 import json
 import os
 import pathlib
-import sys
-import unittest
 from tempfile import TemporaryDirectory
 from typing import Any, Dict
 
 from jsonschema import ValidationError
+from pep440 import is_canonical  # type: ignore
 
+from nbformat import __version__ as nbf_version
 from nbformat import current_nbformat, read, write, writes
 from nbformat.reader import get_version
 from nbformat.validator import isvalid
@@ -20,6 +20,10 @@ from nbformat.validator import isvalid
 from .base import TestsBase
 
 
+def test_canonical_version():
+    assert is_canonical(nbf_version)
+
+
 class TestAPI(TestsBase):
     def test_read(self):
         """Can older notebooks be opened and automatically converted to the current
@@ -54,10 +58,6 @@ class TestAPI(TestsBase):
             write(nb, dest)
             assert os.path.isfile(dest)
 
-    @unittest.skipIf(
-        sys.version_info < (3, 6, 0),
-        "python versions 3.5 and lower don't support opening pathlib.Path objects",
-    )
     def test_read_write_pathlib_object(self):
         """read() and write() take path-like objects such as pathlib objects"""
         path = pathlib.Path(self._get_files_path()) / "test4.ipynb"
